//
//  TemperatureSensorController.m
//  TemperatureSensor
//
//  Created by Phidgets on 2015-07-17.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "TemperatureSensorController.h"


@implementation TemperatureSensorController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch)
    {
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetTemperatureSensor_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetTemperatureSensor_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetTemperatureSensor_setOnTemperatureChangeHandler(ch, gotTemperatureChange, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotTemperatureChange(PhidgetTemperatureSensorHandle phid, void *context, double temperature){
        [(__bridge id)context performSelectorOnMainThread:@selector(onTemperatureChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:temperature]
                                            waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    int heightChange = 0;
    uint32 minDataInterval, maxDataInterval, dataInterval;
    double minTemperatureChangeTrigger,maxTemperatureChangeTrigger, temperatureChangeTrigger;
    Phidget_ChannelSubclass channelSubclass;
    
    //Get information from channel that will allow us to configure the GUI properly
    Phidget_getChannelSubclass((PhidgetHandle)ch, &channelSubclass);
    PhidgetTemperatureSensor_getMinDataInterval(ch, &minDataInterval);
    PhidgetTemperatureSensor_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetTemperatureSensor_getDataInterval(ch, &dataInterval);
    PhidgetTemperatureSensor_getMinTemperatureChangeTrigger(ch, &minTemperatureChangeTrigger);
    PhidgetTemperatureSensor_getMaxTemperatureChangeTrigger(ch, &maxTemperatureChangeTrigger);
    PhidgetTemperatureSensor_getTemperatureChangeTrigger(ch, &temperatureChangeTrigger);
    
    //Adjust GUI based on infromation from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataIntervalBox setHidden:YES];
    [changeTriggerBox setHidden:YES];
    [thermocoupleBox setHidden:YES];
    [rtdBox setHidden:YES];
    [settingsLabel setHidden:NO];
    [stackBox setHidden:NO];
    [dataBox setHidden:NO];
    [dataIntervalBox setHidden:NO];
    
    //Configure the data interval slider with infromation read from channel
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    
    //Check if device has configurable change triger
    if(minTemperatureChangeTrigger == maxTemperatureChangeTrigger){
        [changeTriggerBox setHidden:YES];
    }
    else{
        [changeTriggerBox setHidden:NO];
        //Configure the change trigger slider with information read from channel
        [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f ºC",maxTemperatureChangeTrigger]];
        [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f ºC",minTemperatureChangeTrigger]];
        [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f ºC", temperatureChangeTrigger]];
        [changeTriggerSlider setMaxValue:maxTemperatureChangeTrigger];
        [changeTriggerSlider setMinValue:minTemperatureChangeTrigger];
        [changeTriggerSlider setIntValue:temperatureChangeTrigger];
    }
    
    switch(channelSubclass){
        case PHIDCHSUBCLASS_TEMPERATURESENSOR_RTD:{
            PhidgetTemperatureSensor_RTDType rtdType;
            Phidget_RTDWireSetup rtdWireSetup;
            PhidgetTemperatureSensor_getRTDType(ch, &rtdType);
            PhidgetTemperatureSensor_getRTDWireSetup(ch, &rtdWireSetup);
            [rtdBox setHidden:NO];
            
            if(rtdType == RTD_TYPE_PT100_3850)
                [rtdTypeCombo selectItemAtIndex:0];
            else if(rtdType == RTD_TYPE_PT1000_3850)
                [rtdTypeCombo selectItemAtIndex:1];
            else if(rtdType == RTD_TYPE_PT100_3920)
                [rtdTypeCombo selectItemAtIndex:2];
            else if(rtdType == RTD_TYPE_PT1000_3920)
                [rtdTypeCombo selectItemAtIndex:3];
            
            if(rtdWireSetup == RTD_WIRE_SETUP_2WIRE)
                [rtdSetupCombo selectItemAtIndex:0];
            else if(rtdWireSetup == RTD_WIRE_SETUP_3WIRE)
                [rtdSetupCombo selectItemAtIndex:1];
            else if(rtdWireSetup == RTD_WIRE_SETUP_4WIRE)
                [rtdSetupCombo selectItemAtIndex:2];
        }
            break;
        case PHIDCHSUBCLASS_TEMPERATURESENSOR_THERMOCOUPLE:{
            PhidgetTemperatureSensor_ThermocoupleType thermocoupleType;
            PhidgetTemperatureSensor_getThermocoupleType(ch, &thermocoupleType);
            [stackBox setHidden:NO];
            [thermocoupleBox setHidden:NO];
            //Set to K-Type by default
            if(thermocoupleType == THERMOCOUPLE_TYPE_K)
                [thermocoupleTypeCombo selectItemAtIndex:1];
            else if(thermocoupleType == THERMOCOUPLE_TYPE_E)
                [thermocoupleTypeCombo selectItemAtIndex:1];
            else if(thermocoupleType == THERMOCOUPLE_TYPE_J)
                [thermocoupleTypeCombo selectItemAtIndex:0];
            else
                [thermocoupleTypeCombo selectItemAtIndex:3];
        }
            break;
        default:
            break;
    }
    
    //Adjusting view height
    NSRect frame = [mainWindow frame];
    if(![thermocoupleBox isHidden])
        heightChange =  [mainWindow maxSize].height + thermocoupleBox.frame.size.height;
    else if(![rtdBox isHidden])
        heightChange = [mainWindow maxSize].height + rtdBox.frame.size.height;
    else
        heightChange = [mainWindow maxSize].height;
    frame.size.height = heightChange;
    [mainWindow setFrame:frame display:NO animate:NO];
    [mainWindow center];

}


- (void)onDetachHandler{
    //Reset
    lastBox = nil;
    [settingsLabel setHidden:YES];
    [temperatureText setStringValue:@""];
    
    //Taking items out of view
    [dataBox setHidden:YES];
    [dataIntervalBox setHidden:YES];
    [changeTriggerBox setHidden:YES];
    [rtdBox setHidden:YES];
    [stackBox setHidden:YES];
    [thermocoupleBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (void)onTemperatureChangeHandler:(NSNumber *)temperature{
    [temperatureText setStringValue:[NSString stringWithFormat:@"%0.3f ºC", [temperature doubleValue]]];
}


#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetTemperatureSensor_setTemperatureChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f ºC", [changeTriggerSlider doubleValue]]];
    
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetTemperatureSensor_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self addError:result];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
    
}

- (IBAction)setThermocoupleType:(id)sender{
    PhidgetReturnCode result;
    if([thermocoupleTypeCombo indexOfSelectedItem] == 0)
        result = PhidgetTemperatureSensor_setThermocoupleType(ch, THERMOCOUPLE_TYPE_J);
    else if([thermocoupleTypeCombo indexOfSelectedItem] == 1)
        result = PhidgetTemperatureSensor_setThermocoupleType(ch, THERMOCOUPLE_TYPE_K);
    else if([thermocoupleTypeCombo indexOfSelectedItem] == 2)
        result = PhidgetTemperatureSensor_setThermocoupleType(ch, THERMOCOUPLE_TYPE_E);
    else if([thermocoupleTypeCombo indexOfSelectedItem] == 3)
        result = PhidgetTemperatureSensor_setThermocoupleType(ch, THERMOCOUPLE_TYPE_T);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK)
        [self addError:result];
}

- (IBAction)setRTDType:(id)sender {
    PhidgetReturnCode result;
    if([rtdTypeCombo indexOfSelectedItem] == 0)
        result = PhidgetTemperatureSensor_setRTDType(ch, RTD_TYPE_PT100_3850);
    else if([rtdTypeCombo indexOfSelectedItem] == 1)
    result = PhidgetTemperatureSensor_setRTDType(ch, RTD_TYPE_PT1000_3850);
    else if([rtdTypeCombo indexOfSelectedItem] == 2)
        result = PhidgetTemperatureSensor_setRTDType(ch, RTD_TYPE_PT100_3920);
    else if([rtdTypeCombo indexOfSelectedItem] == 3)
        result = PhidgetTemperatureSensor_setRTDType(ch, RTD_TYPE_PT1000_3920);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK)
        [self addError:result];
    
}
- (IBAction)setRTDWireSetup:(id)sender {
    PhidgetReturnCode result;
    if([rtdSetupCombo indexOfSelectedItem] == 0)
        result = PhidgetTemperatureSensor_setRTDWireSetup(ch, RTD_WIRE_SETUP_2WIRE);
    else if([rtdSetupCombo indexOfSelectedItem] == 1)
        result = PhidgetTemperatureSensor_setRTDWireSetup(ch, RTD_WIRE_SETUP_3WIRE);
    else if([rtdSetupCombo indexOfSelectedItem] == 2)
        result = PhidgetTemperatureSensor_setRTDWireSetup(ch, RTD_WIRE_SETUP_4WIRE);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK)
        [self addError:result];
    
}

@end
